package v1

import (
	"access-control-monitor/parser"
	"context"
	"github.com/jaryhe/gopkgs/database"
	"go.uber.org/zap"
	"access-control-monitor/pb"
	model "access-control-monitor/model/v1"
	"access-control-monitor/pb/v1"
	"time"
	"github.com/jaryhe/gopkgs/logger"
)


func ProvincialTeamTask() {
	if !parser.Conf.UseProvincial {
		return
	}
	team := model.TeamMasterInfo{}
	where := map[string]interface{}{
		"ProvincialVerifyStatus !=": 204,
		"ProvincialVerifyStatus is": "NULL",
	}
	page := int32(0)
	for {
		list, _ := team.List(database.DB(), nil, where,  page, 10)
		for _, v := range list {

			if v.ProvincialVerifyID == "" {
				continue
			}

			mreq := v1.ProvincialVerifyDataRequest{Id:v.ProvincialVerifyID}
			mreply, err := pb.Provincial.ProvincialVerifyData(context.Background(), &mreq)

			if err != nil {
				logger.Error("ProvincialTask",
					zap.String("call", "pb.Thirdparty.ProvincialVerifyData"),
					zap.String("param", v.ProvincialVerifyID),
					zap.String("error",err.Error()))
				time.Sleep(1*time.Second)
				continue
			}
			p := model.TeamMasterInfo{}
			where := map[string]interface{}{
				"ID": v.ID,
			}
			values := map[string]interface{}{
				"ProvincialVerifyStatus":mreply.State,
				"ProvincialVerifyCode":mreply.Result,
				"ProvincialVerifyInfo":mreply.AllMessage,
			}
			err = p.Update(database.DB(), where, values)
			logger.Error("ProvincialTask",
				zap.String("mysql", "update"),
				zap.String("error",err.Error()))
			time.Sleep(1*time.Second)
		}
		if len(list) < 10 {
			page = 0
			time.Sleep(30*time.Second)
			continue
		}
		page += 1
	}
}

func ProvincialStaffTask() {
	if !parser.Conf.UseProvincial {
		return
	}
	staff := model.ProjectWorkInfo{}
	where := map[string]interface{}{
		"ProvincialVerifyStatus !=": 204,
		"ProvincialVerifyStatus is": "NULL",
	}
	page := int32(0)
	for {
		list, _ := staff.List(database.DB(), nil, where,  page, 10)
		for _, v := range list {

			if v.ProvincialVerifyID == "" {
				continue
			}

			mreq := v1.ProvincialVerifyDataRequest{Id:v.ProvincialVerifyID}
			mreply, err := pb.Provincial.ProvincialVerifyData(context.Background(), &mreq)

			if err != nil {
				logger.Error("ProvincialTask",
					zap.String("call", "pb.Thirdparty.ProvincialVerifyData"),
					zap.String("param", v.ProvincialVerifyID),
					zap.String("error",err.Error()))
				time.Sleep(1*time.Second)
				continue
			}
			p := model.ProjectWorkInfo{}
			where := map[string]interface{}{
				"ID": v.Id,
			}
			values := map[string]interface{}{
				"ProvincialVerifyStatus":mreply.State,
				"ProvincialVerifyCode":mreply.Result,
				"ProvincialVerifyInfo":mreply.AllMessage,
			}
			err = p.Update(database.DB(), where, values)
			logger.Error("ProvincialTask",
				zap.String("mysql", "update"),
				zap.String("error",err.Error()))
			time.Sleep(1*time.Second)
		}
		if len(list) < 10 {
			page = 0
			time.Sleep(30*time.Second)
			continue
		}
		page += 1
	}

}
