// Copyright 2019 autocareai.com. All rights reserved.
// Use of this source code is governed by autocareai.com.

package v1

import (
	"access-control-monitor/consts"
	"access-control-monitor/errors"
	"access-control-monitor/model/v1"
	param_v1 "access-control-monitor/param/v1"
	"access-control-monitor/parser"
	"access-control-monitor/utils"
	"context"
	"encoding/base64"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"time"

	"access-control-monitor/pb"
	pb_v1 "access-control-monitor/pb/v1"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/util"
)

const CRYPTO_KEY = "2D29BP43U3Y1B4N6REQW3F319DD23454"

func certEncrypt(cert string) string {
	cryPasswd, _ := util.AesEncrypt(cert, CRYPTO_KEY)
	ret := base64.StdEncoding.EncodeToString(cryPasswd)
	return ret
}

func certDecrypt(cert string) string {
	bytes, _ := base64.StdEncoding.DecodeString(cert)
	ret, _ := util.AesDecrypt(bytes, []byte(CRYPTO_KEY))
	return string(ret)
}

func StaffIn(db *gorm.DB, body param_v1.RegisterBody, projectId int64, code string, now time.Time) error {
	p := &model.ProjectWorkInoutInfo{
		ProjectId:    projectId,
		Code:         code,
		IDCardNumber: body.IDCardNumber,
		OccurTime:    now,
		InOut:        1,
		WorkNo:       body.WorkNo,
	}
	if err := p.Insert(db); err != nil {
		return errors.DataBaseError
	}
	return nil
}

func StaffOut(db *gorm.DB, workNo string, idCardNumber string, now time.Time, projectId int64, code string) error {
	p := &model.ProjectWorkInoutInfo{
		ProjectId:    projectId,
		Code:         code,
		IDCardNumber: idCardNumber,
		OccurTime:    now,
		InOut:        2,
		WorkNo:       workNo,
	}
	if err := p.Insert(db); err != nil {
		return errors.DataBaseError
	}
	return nil
}

func PostStaffToProvincial(body param_v1.RegisterBody, code string, oriCert string) (string, error) {
	mreq := pb_v1.ProvincialPostStaffRequest{
		Code: code,
		TeamSysNo: int64(body.TeamSysNo),
		TeamName:  body.TeamName,
		CorpName:  body.CorpName,
		CorpCode:  body.CorpCode,
		WorkerName:body.WorkName,
		IsTeamLeader:body.IsTeamLeader,
		IdCardType:int32(body.IDCardType),
		IdCardNumber:body.IDCardNumber,//todo
		Age:int32(body.Age),
		Gender:body.Gender,
		Nation:consts.NationIntStr[body.Nation],
		Address:body.Address,
		HeadImage:body.HeadImage,
		PoliticsType:int32(body.PoliticsType),
		CultureLevelType:int32(body.CultureLevelType),
		GrantOrg:body.GrantOrg,
		HasContract:body.HasContract,
		Mobile:body.Mobile,
		WorkType:body.WorkType,
		NativePlace:body.NativePlace,
	}
	//todo
	var err error
	mreq.IdCardNumber, err = utils.ProvincialEncrypt([]byte(body.IDCardNumber), []byte(parser.Conf.ThirdParty.Provincial.AppSecret))
	if err != nil {
		logger.Error("ProvincialEncrypt",
			zap.String("err", err.Error()))
	}
	mreply, err := pb.Provincial.ProvincialPostStaff(context.Background(), &mreq)
	if err != nil {
		logger.Error("ProvincialPostStaff",
			zap.String("err", err.Error()))
		return "", err
	}
	return mreply.Id, nil

}

func checkTeam(no int, projectId int64, name string) error {
	tmp := model.TeamMasterInfo{}
	where := map[string]interface{}{
		"TeamName": name,
		"ProjectId": projectId,
		"TeamSysNo": no,
	}
	err := tmp.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return errors.DataBaseError
	}

	if tmp.ID == 0 {
		return status.Error(10002, "班组名或班组号不存在")
	}
	return nil
}

// 工人注册
func StaffRegister(body param_v1.RegisterBody, projectId int64, code string) error {
	now := time.Now()
	oriCert := body.IDCardNumber
	body.IDCardNumber = certEncrypt(body.IDCardNumber)

	p := &model.ProjectWorkInfo{
		Code:             code,
		ProjectId:        projectId,
		CorpName:         body.CorpName,
		CorpCode:         body.CorpCode,
		TeamName:         body.TeamName,
		TeamSysNo:        body.TeamSysNo,
		WorkNo:           body.WorkNo,
		WorkName:         body.WorkName,
		IDCardType:       body.IDCardType,
		IDCardNumber:     body.IDCardNumber,
		Age:              body.Age,
		Nation:           consts.NationIntMap[body.Nation],
		Address:          body.Address,
		HeadImage:        body.HeadImage,
		PoliticsType:     body.PoliticsType,
		CultureLevelType: body.CultureLevelType,
		GrantOrg:         body.GrantOrg,
		NativePlace:      body.NativePlace,
		Mobile:           body.Mobile,
		IsTeamLeader:     body.IsTeamLeader,
		Gender:           body.Gender,
		HasContract:      body.HasContract,
		CreatedAt:        now,
		UpdatedAt:        now,
		WorkType:         body.WorkType,
		Sn:               body.Sn,
	}
	// 检查工号
	old := model.ProjectWorkInfo{}
	where := map[string]interface{}{
		"WorkNo":    body.WorkNo,
		"ProjectId": projectId,
	}
	err := old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return errors.DataBaseError
	}
	if old.Id > 0 {
		return status.Error(10002, "工号已存在，请使用更新接口更新")
	}

	// 检查身份证
	old = model.ProjectWorkInfo{}
	where = map[string]interface{}{
		"IDCardNumber":    body.IDCardNumber,
		"ProjectId": projectId,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return errors.DataBaseError
	}
	if old.Id > 0 {
		return status.Error(10002, "身份证号已存在")
	}

	// 检查班组
	if err := checkTeam(body.TeamSysNo, projectId, body.TeamName); err != nil {
		return err
	}

	// TODO
	if parser.Conf.UseProvincial {
		provincialId, err := PostStaffToProvincial(body, code, oriCert)
		if err != nil {
			return err
		}
		p.ProvincialVerifyID = provincialId
	}

	db := database.DB().Begin()
	err = p.Insert(db)
	if err != nil {
		db.Rollback()
		return errors.DataBaseError
	}
	err = StaffIn(db, body, projectId, code, now)
	if err != nil {
		db.Rollback()
		return errors.DataBaseError
	}
	db.Commit()
	return nil

}

func StaffDelete(workNo []string, projectId int64, code string) error {

	old := model.ProjectWorkInfo{}
	where := map[string]interface{}{
		"WorkNo in": workNo,
		"ProjectId": projectId,
	}
	db := database.DB().Begin()

	infos, err := model.GetIDCardNumberByWorkNo(db, projectId, workNo)
	if err != nil {
		db.Rollback()
		return errors.DataBaseError
	}

	err = old.Del(db, where)
	if err != nil {
		db.Rollback()
		return errors.DataBaseError
	}

	now := time.Now()
	for _, v := range infos {
		err := StaffOut(db, v.WorkNo, v.IDCardNumber, now, projectId, code)
		if err != nil {
			db.Rollback()
			return errors.DataBaseError
		}
	}
	db.Commit()
	return nil
}

func StaffUpdate(body param_v1.RegisterBody, projectId int64, code string) error {
	p := &model.ProjectWorkInfo{}
	body.IDCardNumber = certEncrypt(body.IDCardNumber)
	old := model.ProjectWorkInfo{}
	where := map[string]interface{}{
		"WorkNo":    body.WorkNo,
		"ProjectId": projectId,
	}

	err := old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return errors.DataBaseError
	}

	// 不存在
	if old.Id == 0 {
		return status.Error(10002, "没有找到该工人")
	}

	if old.IDCardNumber != body.IDCardNumber {
		tmp := model.ProjectWorkInfo{}
		where := map[string]interface{}{
			"WorkNo !=":    body.WorkNo,
			"ProjectId": projectId,
			"IDCardNumber":    body.IDCardNumber,
		}
		err = tmp.Find(database.DB(), where)
		if err != nil && err != gorm.ErrRecordNotFound {
			return errors.DataBaseError
		}
		if tmp.Id > 0 {
			return status.Error(10002, "身份证已存在")
		}
	}

	// 检查班组
	if err := checkTeam(body.TeamSysNo, projectId, body.TeamName); err != nil {
		return err
	}

	// 已存在则更新
	where["ID"] = old.Id
	values := map[string]interface{}{
		"Code":             code,
		"ProjectId":        projectId,
		"CorpName":         body.CorpName,
		"CorpCode":         body.CorpCode,
		"TeamName":         body.TeamName,
		"TeamSysNo":        body.TeamSysNo,
		"WorkNo":           body.WorkNo,
		"WorkName":         body.WorkName,
		"IsTeamLeader":     body.IsTeamLeader,
		"IDCardType":       body.IDCardType,
		"IDCardNumber":     body.IDCardNumber,
		"Age":              body.Age,
		"Gender":           body.Gender,
		"Nation":           consts.NationIntMap[body.Nation],
		"Address":          body.Address,
		"HeadImage":        body.HeadImage,
		"PoliticsType":     body.PoliticsType,
		"CultureLevelType": body.CultureLevelType,
		"GrantOrg":         body.GrantOrg,
		"NativePlace":      body.NativePlace,
		"Mobile":           body.Mobile,
		"HasContract":      body.HasContract,
		"WorkType":         body.WorkType,
	}

	err = p.Update(database.DB(), where, values)
	if err != nil {
		return errors.DataBaseError
	}

	return nil

}
