// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package model

import (
	"access-control-monitor/errors"
	"fmt"
	"time"

	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	
	"go.uber.org/zap"
	json2 "encoding/json"
)

// 替换encoding/json包
type JsonStruct struct {

}
func (p *JsonStruct)MarshalToString(v interface{}) (string, error) {
	bytes, err := json2.Marshal(v)
	return string(bytes), err
}
func (p *JsonStruct)Marshal(v interface{}) ([]byte, error) {
	bytes, err := json2.Marshal(v)
	return bytes, err
}
func (p *JsonStruct)Unmarshal(bytes []byte, v interface{}) (error) {
	err := json2.Unmarshal(bytes, v)
	return err
}
var json = &JsonStruct{}

type WorkerAttendanceInfo struct {
	ID         int64  `gorm:"column:ID;primary_key"`
	Code string `gorm:"column:Code" json:"code"`
	TeamSysNo int `gorm:"column:TeamSysNo" json:"team_sys_no"`
	TeamName string `gorm:"column:TeamName" json:"team_name"`
	WorkName string `gorm:"column:WorkName" json:"work_name"`
	IDCardType int `gorm:"column:IDCardType" json:"id_card_type"`
	IDCardNumber string `gorm:"column:IDCardNumber" json:"id_card_number"`
	Date time.Time `gorm:"column:Date" json:"date"`
	Direction int `gorm:"column:Direction" json:"direction"`
	Temperature float64 `gorm:"column:Temperature" json:"temperature"`
	ImageURL string `gorm:"column:ImageURL" json:"image_url"`
	AttendType int `gorm:"column:AttendType" json:"attend_type"`
	Lng float64 `gorm:"column:Lng" json:"lng"`
	Lat float64 `gorm:"column:Lat" json:"lat"`
	ProjectId  int64  `gorm:"column:ProjectId" json:"project_id"`
	WorkNo     string `gorm:"column:WorkNo" json:"work_no"`
	Sn string `gorm:"column:Sn" json:"sn"`
	ProvincialVerifyStatus int32 `gorm:"column:ProvincialVerifyStatus" json:"provincial_verify_status"`
	ProvincialVerifyCode int32 `gorm:"column:ProvincialVerifyCode" json:"provincial_verify_code"`
	ProvincialVerifyInfo string `gorm:"column:ProvincialVerifyInfo" json:"provincial_verify_info"`
	ProvincialVerifyID string `gorm:"column:ProvincialVerifyID" json:"provincial_verify_id"`
	TabType int `gorm:"-"` // 用于判断表0 t_staff_attendance, 1-12 t_staff_attendance_monthx, 13 t_staff_attendance_day
}

const (
	AttendanceTabDay = iota
	AttendanceTabMonth1
	AttendanceTabMonth2
	AttendanceTabMonth3
	AttendanceTabMonth4
	AttendanceTabMonth5
	AttendanceTabMonth6
	AttendanceTabMonth7
	AttendanceTabMonth8
	AttendanceTabMonth9
	AttendanceTabMonth10
	AttendanceTabMonth11
	AttendanceTabMonth12
)

func (p WorkerAttendanceInfo) TableName() string {

		if p.TabType >=AttendanceTabMonth1 && p.TabType <= AttendanceTabMonth12 {
			return fmt.Sprintf("db_smart_attendance.WorkerAttendanceInfoMonth%d", p.TabType)
		}
		return "db_smart_attendance.WorkerAttendanceInfoDay"
}

// Insert 插入一条记录
func (p *WorkerAttendanceInfo) Insert(db *gorm.DB) error {
	//p.UpdatedAt = timeNow
	err := db.Table(p.TableName()).Create(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *WorkerAttendanceInfo) Delete(db *gorm.DB, filter map[string]interface{}) error {
	cond, val, err := whereBuild(filter)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "delete from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *WorkerAttendanceInfo) Save(db *gorm.DB) error {

	err := db.Table(p.TableName()).Save(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "save " +p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *WorkerAttendanceInfo) Update(db *gorm.DB) error {
	err := db.Table(p.TableName()).Model(p).Updates(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "update "+ p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *WorkerAttendanceInfo) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	if filed == nil {
		return errors.ParamsError
	}
	//timeNow := time.Now().Format(consts.TimeSecondLayOut)
	//filed["updated_at"] = timeNow
	err := db.Table(p.TableName()).Updates(filed).Error
	if err != nil {
		fields, _ := json.MarshalToString(filed)
		logger.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *WorkerAttendanceInfo) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Table(p.TableName()).Where(filter).Find(p).Error
	if err != nil {
		fields, _ := json.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *WorkerAttendanceInfo) QueryAll(db *gorm.DB, filter map[string]interface{}, out interface{}) error {
	err := db.Table(p.TableName()).Where(filter).Find(out).Error
	if err != nil {
		fields, _ := json.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

type TeamSys struct {
	TeamSysNo int
}

func GetTeamSysNo(db *gorm.DB, projectId int64, sysNo []int) ([]TeamSys,error) {
	sql := fmt.Sprintf("select TeamSysNo as team_sys_no from db_smart_attendance.TeamMasterInfo where ProjectId=? and TeamSysNo in(?)")
	array := []TeamSys{}
	err := db.Raw(sql, projectId, sysNo).Scan(&array).Error
	if err != nil {
		return nil, err
	}
	return array, nil
}
